/*
 *
 * Copyright (c) 2021 Texas Instruments Incorporated
 *
 * All rights reserved not granted herein.
 *
 * Limited License.
 *
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
 * license under copyrights and patents it now or hereafter owns or controls to make,
 * have made, use, import, offer to sell and sell ("Utilize") this software subject to the
 * terms herein.  With respect to the foregoing patent license, such license is granted
 * solely to the extent that any such patent is necessary to Utilize the software alone.
 * The patent license shall not apply to any combinations which include this software,
 * other than combinations with devices manufactured by or for TI ("TI Devices").
 * No hardware patent is licensed hereunder.
 *
 * Redistributions must preserve existing copyright notices and reproduce this license
 * (including the above copyright notice and the disclaimer and (if applicable) source
 * code license limitations below) in the documentation and/or other materials provided
 * with the distribution
 *
 * Redistribution and use in binary form, without modification, are permitted provided
 * that the following conditions are met:
 *
 * *       No reverse engineering, decompilation, or disassembly of this software is
 * permitted with respect to any software provided in binary form.
 *
 * *       any redistribution and use are licensed by TI for use only with TI Devices.
 *
 * *       Nothing shall obligate TI to provide you with source code for the software
 * licensed and provided to you in object code.
 *
 * If software source code is provided to you, modification and redistribution of the
 * source code are permitted provided that the following conditions are met:
 *
 * *       any redistribution and use of the source code, including any resulting derivative
 * works, are licensed by TI for use only with TI Devices.
 *
 * *       any redistribution and use of any object code compiled from the source code
 * and any resulting derivative works, are licensed by TI for use only with TI Devices.
 *
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers
 *
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * DISCLAIMER.
 *
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include<unistd.h>
#include <VX/vx.h>
#include <TI/tivx.h>
#include <TI/tivx_config.h>
#include <string.h>
#include <TI/tivx_task.h>
#include <TI/tivx_event.h>
#include "math.h"
#include <limits.h>
#include <utils/mem/include/app_mem.h>


#define TIVX_TARGET_DEFAULT_STACK_SIZE      (256U * 1024U)
#define TIVX_TARGET_DEFAULT_TASK_PRIORITY1   (8u)

#define DSS_M2M_NUM_CH                              (1U)
#define DSS_M2M_NUM_CH_MAX                          (4U)

/* Common Configurations across channels */
#define DSS_M2M_WB_PIPE_INST_ID                     (0U)
#define DSS_M2M_PIPE_NUM                            (1U)
#define DSS_M2M_PIPE_INST_ID                        (3U)
/* Currently Only Overlay2 can be used for M2M operations,
   this can be changed through DSS initialization API available in vision_apps */
#define DSS_M2M_OVERLAY_ID                          (3U)

/* Channel 0 configurations */
#define DSS_M2M_CH0_IN_FRAME_FORMAT                 (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH0_IN_FRAME_WIDTH                  (1280U)
#define DSS_M2M_CH0_IN_FRAME_HEIGHT                 (1285U*4U)
#define DSS_M2M_CH0_IN_FRAME_BPP                    (2U)
#define DSS_M2M_CH0_IN_FRAME_PITCH                  (DSS_M2M_CH0_IN_FRAME_WIDTH * \
                                                     DSS_M2M_CH0_IN_FRAME_BPP * 4)
#define DSS_M2M_CH0_OUT_FRAME_FORMAT                (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH0_OUT_FRAME_WIDTH                 (1280U)
#define DSS_M2M_CH0_OUT_FRAME_HEIGHT                (1285U)

/* Channel 1 configurations */
#define DSS_M2M_CH1_IN_FRAME_FORMAT                 (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH1_IN_FRAME_WIDTH                  (1280U)
#define DSS_M2M_CH1_IN_FRAME_HEIGHT                 (1285U*4U)
#define DSS_M2M_CH1_IN_FRAME_BPP                    (2U)
#define DSS_M2M_CH1_IN_FRAME_PITCH                  (DSS_M2M_CH1_IN_FRAME_WIDTH * \
                                                     DSS_M2M_CH1_IN_FRAME_BPP * 4)
#define DSS_M2M_CH1_OUT_FRAME_FORMAT                (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH1_OUT_FRAME_WIDTH                 (1280U)
#define DSS_M2M_CH1_OUT_FRAME_HEIGHT                (1285U)

/* Channel 2 configurations */
#define DSS_M2M_CH2_IN_FRAME_FORMAT                 (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH2_IN_FRAME_WIDTH                  (1280U)
#define DSS_M2M_CH2_IN_FRAME_HEIGHT                 (1285U*4U)
#define DSS_M2M_CH2_IN_FRAME_BPP                    (2U)
#define DSS_M2M_CH2_IN_FRAME_PITCH                  (DSS_M2M_CH2_IN_FRAME_WIDTH * \
                                                     DSS_M2M_CH2_IN_FRAME_BPP * 4)
#define DSS_M2M_CH2_OUT_FRAME_FORMAT                (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH2_OUT_FRAME_WIDTH                 (1280U)
#define DSS_M2M_CH2_OUT_FRAME_HEIGHT                (1285U)

/* Channel 3 configurations */
#define DSS_M2M_CH3_IN_FRAME_FORMAT                 (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH3_IN_FRAME_WIDTH                  (1280U)
#define DSS_M2M_CH3_IN_FRAME_HEIGHT                 (1285U*4U)
#define DSS_M2M_CH3_IN_FRAME_BPP                    (2U)
#define DSS_M2M_CH3_IN_FRAME_PITCH                  (DSS_M2M_CH3_IN_FRAME_WIDTH * \
                                                     DSS_M2M_CH3_IN_FRAME_BPP * 4)
#define DSS_M2M_CH3_OUT_FRAME_FORMAT                (VX_DF_IMAGE_UYVY)
#define DSS_M2M_CH3_OUT_FRAME_WIDTH                 (1280U)
#define DSS_M2M_CH3_OUT_FRAME_HEIGHT                (1285U)

#define DSS_M2M_NODE_NAME_LEN_MAX                   (100U)


#define LDC_BLOCK_WIDTH                             (64)
#define LDC_BLOCK_HEIGHT                            (64)
#define LDC_PIXEL_PAD                               (0)


/* Preloaded image buffers */
/* 1920 x 1080 buffers */
extern uint32_t gTiovxCtDisplayArrayBGR888[1555200];
extern uint32_t gTiovxCtDisplayArrayYUV420NV12[777600];
extern uint32_t gTiovxCtDisplayArrayYUV422[1036800];

typedef struct {
    uint32_t taskId;
    uint32_t instId;
    uint32_t numPipe;
    uint32_t pipeId[TIVX_DISPLAY_M2M_MAX_PIPE];
    uint32_t overlayId;
    vx_df_image inFmt;
    uint32_t inWidth;
    uint32_t inHeight;
    uint32_t inBpp;
    uint32_t inPitch;
    vx_df_image outFmt;
    uint32_t outWidth;
    uint32_t outHeight;
    uint32_t posX;
    uint32_t posY;
    tivx_event eventHandle_TaskFinished;
    tivx_task taskHandle_m2m;
    tivx_task_create_params_t taskParams_m2m;
    char nodeName[DSS_M2M_NODE_NAME_LEN_MAX];
    uint32_t iterationCnt;
} tivx_display_m2m_test_params_t;

static vx_context context;
static uint32_t gLoop_cnt;
tivx_display_m2m_test_params_t gTestParams[DSS_M2M_NUM_CH_MAX];
extern uint8_t *src_buf, *dest_buf;

static void tivxTask_m2m(void *app_var)
{
    vx_node m2m_node = 0, ldc_node = 0;
    vx_image in_image = 0;
    vx_image out_image = 0;
    vx_image ldc_out_img = 0;
    vx_user_data_object m2m_config;
    tivx_display_m2m_params_t local_m2m_config;
    vx_imagepatch_addressing_t image_addr;
    vx_rectangle_t rect;
    vx_graph m2m_graph = 0;
    tivx_display_m2m_test_params_t *testParams =
                    (tivx_display_m2m_test_params_t *)app_var;
    uint32_t status = 0;
    //vx_bool dss_m2m_prms_replicate[] = {vx_false_e, vx_false_e, vx_true_e};

    vx_imagepatch_addressing_t image_addr_from_file, image_addr_from_file2;
    vx_rectangle_t rect_from_file;
    vx_map_id map_id1, map_id2;
    void *data_ptr1, *data_ptr2;
    uint32_t wbFrmCnt;
    tivx_vpac_ldc_params_t ldc_params;
    vx_user_data_object ldc_config;
    tivx_vpac_ldc_region_params_t ldc_region_params;
    vx_user_data_object ldc_region_config;

    m2m_graph = vxCreateGraph(context);

    printf("Graph %d: created...\n", testParams->taskId);

    /* allocate Input and Output frame refs */
    in_image = vxCreateImage(context,
                                              testParams->inWidth,
                                              testParams->inHeight,
                                              testParams->inFmt);
    out_image = vxCreateImage(context,
                                               testParams->outWidth,
                                               testParams->outHeight,
                                               testParams->outFmt);

    vx_object_array out_img_array = vxCreateObjectArray(context, (vx_reference)out_image, 4);
    vxReleaseImage(&out_image);

    out_image = vxCreateImage(context,
                                               testParams->outWidth,
                                               testParams->outHeight,
                                               VX_DF_IMAGE_NV12);

    vx_object_array ldc_out_img_array = vxCreateObjectArray(context, (vx_reference)out_image, 4);
    vxReleaseImage(&out_image);

    out_image = (vx_image)vxGetObjectArrayItem(out_img_array, 0);
    ldc_out_img = (vx_image)vxGetObjectArrayItem(ldc_out_img_array, 0);

    printf("Graph %d: input and output images created...\n", testParams->taskId);

    image_addr.dim_x    = testParams->inWidth;
    image_addr.dim_y    = testParams->inHeight;
    image_addr.stride_x = testParams->inBpp;
    image_addr.stride_y = testParams->inPitch;
    image_addr.scale_x  = VX_SCALE_UNITY;
    image_addr.scale_y  = VX_SCALE_UNITY;
    image_addr.step_x   = 1;
    image_addr.step_y   = 1;
    rect.start_x        = 0;
    rect.start_y        = 0;
    rect.end_x          = testParams->inWidth;
    rect.end_y          = testParams->inHeight;
    printf("Graph %d: input and output images created 2 ...\n", testParams->taskId);
    printf("%d\n", __LINE__);
    tivxTaskWaitMsecs(1000);
    /* Copy reference input image to input buffer */
    if (testParams->inFmt == VX_DF_IMAGE_RGB)
    {
        vxCopyImagePatch(in_image,
                         &rect,
                         0,
                         &image_addr,
                         (void *)gTiovxCtDisplayArrayBGR888,
                         VX_WRITE_ONLY,
                         VX_MEMORY_TYPE_HOST);
    }
    else if (testParams->inFmt == VX_DF_IMAGE_YUYV)
    {
        vxCopyImagePatch(in_image,
                         &rect,
                         0,
                         &image_addr,
                         (void *)gTiovxCtDisplayArrayYUV422,
                         VX_WRITE_ONLY,
                         VX_MEMORY_TYPE_HOST);
    }
    else if (testParams->inFmt == VX_DF_IMAGE_UYVY)
    {
        rect_from_file.start_x        = 0;
        rect_from_file.start_y        = 0;
        rect_from_file.end_x          = testParams->inWidth;
        rect_from_file.end_y          = testParams->inHeight;
        /* Update input buffer here for other formats */
        status = vxMapImagePatch(in_image,
                                    &rect_from_file,
                                    0,
                                    &map_id1,
                                    &image_addr_from_file,
                                    &data_ptr1,
                                    VX_WRITE_ONLY,
                                    VX_MEMORY_TYPE_HOST,
                                    VX_NOGAP_X);
        if(status != 0)
        {
            printf("vxMapImagePatch failed. \n");
        }
        printf("%d\n", __LINE__);
        memcpy(data_ptr1, &src_buf[1280U*2U*testParams->taskId], 1280U*2U*(1285U*4U - testParams->taskId));

    }
    printf("%d\n", __LINE__);
    /* DSS M2M initialization */
    tivx_display_m2m_params_init(&local_m2m_config);
    local_m2m_config.instId     = testParams->instId;
    /* Only one pipeline is supported */
    local_m2m_config.numPipe    = testParams->numPipe;
    local_m2m_config.pipeId[0U] = testParams->pipeId[0U];
    local_m2m_config.overlayId  = testParams->overlayId;

    local_m2m_config.enable_deiniterleave = 1u;

    m2m_config = vxCreateUserDataObject(context,
                                        "tivx_display_m2m_params_t",
                                        sizeof(tivx_display_m2m_params_t),
                                        &local_m2m_config);
    printf("%d\n", __LINE__);
    m2m_node = tivxDisplayM2MNode(m2m_graph, m2m_config, in_image, out_image);
    printf("%d\n", __LINE__);
    vxSetNodeTarget(m2m_node, VX_TARGET_STRING, &testParams->nodeName[0U]);
    printf("%d\n", __LINE__);

    {
        ldc_config = vxCreateUserDataObject(context, "tivx_vpac_ldc_params_t", sizeof(tivx_vpac_ldc_params_t), NULL);
        status = vxGetStatus((vx_reference)ldc_config);

        if(status == VX_SUCCESS)
        {
            vxSetReferenceName((vx_reference)ldc_config, "ldc_node_config");

            tivx_vpac_ldc_params_init(&ldc_params);
            ldc_params.luma_interpolation_type = 1;

            status = vxCopyUserDataObject(ldc_config, 0,
                                sizeof(tivx_vpac_ldc_params_t),
                                &ldc_params,
                                VX_WRITE_ONLY,
                                VX_MEMORY_TYPE_HOST);
            if(status != VX_SUCCESS)
            {
                printf("[LDC-MODULE] Unable to copy ldc config params into buffer! \n");
            }
        }
        else
        {
            printf("[LDC-MODULE] Unable to create ldc params object! \n");
        }

        /* Block Size parameters */
        ldc_region_params.out_block_width  = LDC_BLOCK_WIDTH;
        ldc_region_params.out_block_height = LDC_BLOCK_HEIGHT;
        ldc_region_params.pixel_pad        = LDC_PIXEL_PAD;

        ldc_region_config = vxCreateUserDataObject(context, "tivx_vpac_ldc_region_params_t", sizeof(tivx_vpac_ldc_region_params_t),  NULL);
        status = vxGetStatus((vx_reference)ldc_region_config);

        if(status == VX_SUCCESS)
        {
            vxSetReferenceName((vx_reference)ldc_region_config, "ldc_node_region_config");

            status = vxCopyUserDataObject(ldc_region_config, 0,
                                sizeof(tivx_vpac_ldc_region_params_t),
                                &ldc_region_params,
                                VX_WRITE_ONLY,
                                VX_MEMORY_TYPE_HOST);
            if(status != VX_SUCCESS)
            {
                printf("[LDC-MODULE] Unable to copy ldc region params to buffer! \n");
            }
        }

        ldc_node = tivxVpacLdcNode(m2m_graph, ldc_config, NULL,
                                    ldc_region_config,
                                    NULL, /* Back mapping disabled*/
                                    NULL, /* Back mapping disabled*/
                                    NULL,
                                    out_image,
                                    ldc_out_img,
                                    NULL);

        vx_bool replicate[] = { vx_false_e, vx_false_e, vx_false_e, vx_false_e, vx_false_e, vx_false_e, vx_true_e, vx_true_e, vx_false_e};
        vxReplicateNode(m2m_graph, ldc_node, replicate, 9);
    }

    // printf("Added \'%s\' node in graph %d\n", &testParams->nodeName[0U], testParams->taskId);

    printf("Graph %d: verifying...\n", testParams->taskId);

    vxVerifyGraph(m2m_graph);

    // printf("Graph %d: verify done...\n", testParams->taskId);

    {
        extern void timer_print_start(int task_id);
        timer_print_start(testParams->taskId);
    }

    for (wbFrmCnt = 0U ; wbFrmCnt < 1000 ; wbFrmCnt++)
    {
        vxProcessGraph(m2m_graph);
    }
    printf("%d\n", __LINE__);
    {
        extern void timer_print_stop(int task_id);
        timer_print_stop(testParams->taskId);
    }

    {
        for(int i = 0; i < 4; i++)
        {
            out_image = (vx_image)vxGetObjectArrayItem(out_img_array, i);
            rect_from_file.start_x        = 0;
            rect_from_file.start_y        = 0;
            rect_from_file.end_x          = testParams->outWidth;
            rect_from_file.end_y          = testParams->outHeight;
            /* Update input buffer here for other formats */
            status = vxMapImagePatch(out_image,
                                        &rect_from_file,
                                        0,
                                        &map_id2,
                                        &image_addr_from_file2,
                                        &data_ptr2,
                                        VX_WRITE_ONLY,
                                        VX_MEMORY_TYPE_HOST,
                                        VX_NOGAP_X);
            if(status != 0)
            {
                printf("vxMapImagePatch failed 2. \n");
            }
            memcpy(&dest_buf[1280U*1285U*2U*i], data_ptr2, 1280U*1285U*2U);
        }

    }

    vxReleaseNode(&m2m_node);
    vxReleaseImage(&in_image);
    vxReleaseImage(&out_image);
    vxReleaseUserDataObject(&m2m_config);

    vxReleaseNode(&ldc_node);
    vxReleaseUserDataObject(&ldc_region_config);
    vxReleaseUserDataObject(&ldc_config);
    vxReleaseObjectArray(&ldc_out_img_array);

    vxReleaseGraph(&m2m_graph);

    /*Signal the completion of m2m graph processing*/
    tivxEventPost(testParams->eventHandle_TaskFinished);
}


void main_dss_m2m_test()
{
    //context = context_->vx_context_;
    uint32_t taskIdx;
    tivx_display_m2m_test_params_t *testParams;
    uint32_t createTask = 0U;
    uint32_t status = 0;

    /* Create OpenVx Context */
    context = vxCreateContext();
    status = vxGetStatus((vx_reference)context);
    printf("Creating context done status : %d!\n", status);

    printf("Starting Display M2M  Conformance Test...\n");

    /* Initialize global test parameters structure to '0' */
    memset(&gTestParams[0U], 0, sizeof(gTestParams));

    {
        tivxHwaLoadKernels(context);
        //CT_RegisterForGarbageCollection(context, ct_teardown_hwa_kernels, CT_GC_OBJECT);
        printf("Starting Display M2M  Conformance Test 2...\n");
        tivx_set_debug_zone(VX_ZONE_MAX);

        //gLoop_cnt = arg_->loopCount;
        gLoop_cnt = 1;

        for (taskIdx = 0U ; taskIdx < DSS_M2M_NUM_CH ; taskIdx++)
        {
            createTask         = 0U;
            testParams         = &gTestParams[taskIdx];
            testParams->taskId = taskIdx;
            tivxEventCreate(&testParams->eventHandle_TaskFinished);
            testParams->instId     = DSS_M2M_WB_PIPE_INST_ID;
            testParams->numPipe    = DSS_M2M_PIPE_NUM;
            /* Note: Directly assigning as only one pipe is supported currently */
            testParams->pipeId[0U] = DSS_M2M_PIPE_INST_ID;
            testParams->overlayId  = DSS_M2M_OVERLAY_ID;
            testParams->inFmt      = DSS_M2M_CH0_IN_FRAME_FORMAT;
            testParams->iterationCnt = gLoop_cnt;
            switch (taskIdx)
            {
                case 0U:
                    /* Initialize test parameters for task 0 */
                    createTask               = 1U;
                    testParams->inWidth      = DSS_M2M_CH0_IN_FRAME_WIDTH;
                    testParams->inHeight     = DSS_M2M_CH0_IN_FRAME_HEIGHT;
                    testParams->inBpp        = DSS_M2M_CH0_IN_FRAME_BPP;
                    testParams->inPitch      = DSS_M2M_CH0_IN_FRAME_PITCH;
                    testParams->outFmt       = DSS_M2M_CH0_OUT_FRAME_FORMAT;
                    testParams->outWidth     = DSS_M2M_CH0_OUT_FRAME_WIDTH;
                    testParams->outHeight    = DSS_M2M_CH0_OUT_FRAME_HEIGHT;
                    strcpy(&testParams->nodeName[0U], TIVX_TARGET_DISPLAY_M2M1);
                break;
                case 1U:
                    createTask               = 1U;
                    /* Initialize test parameters for task 1 */
                    testParams->inFmt        = DSS_M2M_CH1_IN_FRAME_FORMAT;
                    testParams->inWidth      = DSS_M2M_CH1_IN_FRAME_WIDTH;
                    testParams->inHeight     = DSS_M2M_CH1_IN_FRAME_HEIGHT;
                    testParams->inBpp        = DSS_M2M_CH1_IN_FRAME_BPP;
                    testParams->inPitch      = DSS_M2M_CH1_IN_FRAME_PITCH;
                    testParams->outFmt       = DSS_M2M_CH1_OUT_FRAME_FORMAT;
                    testParams->outWidth     = DSS_M2M_CH1_OUT_FRAME_WIDTH;
                    testParams->outHeight    = DSS_M2M_CH1_OUT_FRAME_HEIGHT;
                    strcpy(&testParams->nodeName[0U], TIVX_TARGET_DISPLAY_M2M2);
                break;
                case 2U:
                    createTask               = 1U;
                    /* Initialize test parameters for task 1 */
                    testParams->inFmt        = DSS_M2M_CH2_IN_FRAME_FORMAT;
                    testParams->inWidth      = DSS_M2M_CH2_IN_FRAME_WIDTH;
                    testParams->inHeight     = DSS_M2M_CH2_IN_FRAME_HEIGHT;
                    testParams->inBpp        = DSS_M2M_CH2_IN_FRAME_BPP;
                    testParams->inPitch      = DSS_M2M_CH2_IN_FRAME_PITCH;
                    testParams->outFmt       = DSS_M2M_CH2_OUT_FRAME_FORMAT;
                    testParams->outWidth     = DSS_M2M_CH2_OUT_FRAME_WIDTH;
                    testParams->outHeight    = DSS_M2M_CH2_OUT_FRAME_HEIGHT;
                    strcpy(&testParams->nodeName[0U], TIVX_TARGET_DISPLAY_M2M3);
                break;
                case 3U:
                    createTask               = 1U;
                    /* Initialize test parameters for task 1 */
                    testParams->inFmt        = DSS_M2M_CH3_IN_FRAME_FORMAT;
                    testParams->inWidth      = DSS_M2M_CH3_IN_FRAME_WIDTH;
                    testParams->inHeight     = DSS_M2M_CH3_IN_FRAME_HEIGHT;
                    testParams->inBpp        = DSS_M2M_CH3_IN_FRAME_BPP;
                    testParams->inPitch      = DSS_M2M_CH3_IN_FRAME_PITCH;
                    testParams->outFmt       = DSS_M2M_CH3_OUT_FRAME_FORMAT;
                    testParams->outWidth     = DSS_M2M_CH3_OUT_FRAME_WIDTH;
                    testParams->outHeight    = DSS_M2M_CH3_OUT_FRAME_HEIGHT;
                    strcpy(&testParams->nodeName[0U], TIVX_TARGET_DISPLAY_M2M4);
                break;
                default:
                break;
            }
            if (createTask == 1U)
            {
                /* Setting up task params for m2m_task */
                tivxTaskSetDefaultCreateParams(&testParams->taskParams_m2m);
                testParams->taskParams_m2m.task_main     = &tivxTask_m2m;
                testParams->taskParams_m2m.app_var       = testParams;
                testParams->taskParams_m2m.stack_ptr     = NULL;
                testParams->taskParams_m2m.stack_size    = TIVX_TARGET_DEFAULT_STACK_SIZE;
                testParams->taskParams_m2m.core_affinity = TIVX_TASK_AFFINITY_ANY;
                testParams->taskParams_m2m.priority      = TIVX_TARGET_DEFAULT_TASK_PRIORITY1;

                printf("Creating Task %d...\n", testParams->taskId);
                /* Create Tasks */
                tivxTaskCreate(&testParams->taskHandle_m2m,
                                    &testParams->taskParams_m2m);
            }
        }

        /* wait here for all tasks to finish */
        printf("Waiting for graphs to finish execution...\n");
        for (taskIdx = 0U ; taskIdx < DSS_M2M_NUM_CH ; taskIdx++)
        {
            testParams = &gTestParams[taskIdx];
            tivxEventWait(testParams->eventHandle_TaskFinished, TIVX_EVENT_TIMEOUT_WAIT_FOREVER);
            printf("Received events from Task%d\n", testParams->taskId);
        }

        /* Delete tasks and sync events */
        for (taskIdx = 0U ; taskIdx < DSS_M2M_NUM_CH ; taskIdx++)
        {
            testParams = &gTestParams[taskIdx];
            tivxTaskDelete(&testParams->taskHandle_m2m);
            tivxEventDelete(&testParams->eventHandle_TaskFinished);
        }

        tivxHwaUnLoadKernels(context);

        tivx_set_debug_zone(VX_ZONE_MAX);
    }

    printf("Display M2M  Conformance Test Finished...\n");
}

